@@ -4,85 +4,29 @@
 */
 
 #include "fcbticket.h"
+#include "fcbreader_p.h"
+#include "fcbutil.h"
 
 #include "logging.h"
 #include "asn1/uperdecoder.h"
 
-#define FCB_READ_CONSTRAINED_INT(Name, Min, Max) \
-    if (Name ## IsSet()) \
-        Name = decoder.readConstrainedWholeNumber(Min, Max)
+constexpr inline auto FCB_TIME_MAX = 1440;
+constexpr inline auto FCB_BEGIN_DATE_MIN = -1;
+constexpr inline auto FCB_END_DATE_MIN = 0;
+constexpr inline auto FCB_DATE_MAX = 370;
 
-#define FCB_READ_UNCONSTRAINED_INT(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readUnconstrainedWholeNumber()
-
-#define FCB_READ_IA5STRING(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readIA5String()
-
-#define FCB_READ_IA5STRING_CONSTRAINED(Name, Min, Max) \
-    if (Name ## IsSet()) \
-        Name = decoder.readIA5String(Min, Max)
-
-#define FCB_READ_UTF8STRING(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readUtf8String()
-
-#define FCB_READ_OCTETSTRING(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readOctetString()
-
-#define FCB_READ_ENUM(Name) \
-    do { if (Name ## IsSet()) { \
-        if constexpr (uperHasExtensionMarker(decltype(Name){})) { \
-            Name = decoder.readEnumeratedWithExtensionMarker<decltype(Name)>(); \
-        } else { \
-            Name = decoder.readEnumerated<decltype(Name)>(); \
-        } \
-    } } while(false)
-
-#define FCB_READ_CUSTOM(Name) \
-    if (Name ## IsSet()) \
-        Name.decode(decoder);
-
-#define FCB_READ_SEQUENCE_OF_CONTRAINED_INT(Name, Min, Max) \
-    if (Name ## IsSet()) \
-        Name = decoder.readSequenceOfConstrainedWholeNumber(Min, Max)
-
-#define FCB_READ_SEQUENCE_OF_UNCONTRAINED_INT(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readSequenceOfUnconstrainedWholeNumber()
-
-#define FCB_READ_SEQUENCE_OF_IA5STRING(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readSequenceOfIA5String()
-
-#define FCB_READ_SEQUENCE_OF_UTF8STRING(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readSequenceOfUtf8String()
-
-#define FCB_READ_SEQUENCE_OF_CUSTOM(Name) \
-    if (Name ## IsSet()) \
-        Name = decoder.readSequenceOf<decltype(Name)::value_type>();
-
-#define FCB_READ_INT_IA5_PAIR(Name, Min, Max) \
-    FCB_READ_CONSTRAINED_INT(Name ## Num, Min, Max); \
-    FCB_READ_IA5STRING(Name ## IA5)
-
-#define FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(Name) \
-    FCB_READ_UNCONSTRAINED_INT(Name ## Num); \
-    FCB_READ_IA5STRING(Name ## IA5)
+constexpr inline auto FCB_PRODUCT_ID_NUM_MAX = 32000;
 
 using namespace KItinerary;
 
-void Fcb::ExtensionData::decode(KItinerary::UPERDecoder &decoder)
+void Fcb::v13::ExtensionData::decode(KItinerary::UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     extensionId = decoder.readIA5String();
     extensionData = decoder.readOctetString();
 }
 
-void Fcb::GeoCoordinateType::decode(UPERDecoder &decoder)
+void Fcb::v13::GeoCoordinateType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_ENUM(geoUnit);
@@ -94,21 +38,21 @@ void Fcb::GeoCoordinateType::decode(UPERDecoder &decoder)
     FCB_READ_ENUM(accuracy);
 }
 
-void Fcb::DeltaCoordinate::decode(UPERDecoder &decoder)
+void Fcb::v13::DeltaCoordinate::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     longitude = decoder.readUnconstrainedWholeNumber();
     latitude = decoder.readUnconstrainedWholeNumber();
 }
 
-void Fcb::IssuingData::decode(UPERDecoder &decoder)
+void Fcb::v13::IssuingData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(securityProvider, 1, 32000);
     FCB_READ_INT_IA5_PAIR(issuer, 1, 32000);
     issuingYear = decoder.readConstrainedWholeNumber(2016, 2269);
     issuingDay = decoder.readConstrainedWholeNumber(1, 366);
-    FCB_READ_CONSTRAINED_INT(issuingTime, 0, 1440);
+    FCB_READ_TIME(issuingTime);
     FCB_READ_UTF8STRING(issuerName);
     specimen = decoder.readBoolean();
     securePaperTicket = decoder.readBoolean();
@@ -122,17 +66,12 @@ void Fcb::IssuingData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(pointOfSale);
 }
 
-QDateTime Fcb::IssuingData::issueingDateTime() const
+QDateTime Fcb::v13::IssuingData::issueingDateTime() const
 {
-    QDate date(issuingYear, 1, 1);
-    date = date.addDays(issuingDay - 1);
-    if (issuingTimeIsSet()) {
-        return QDateTime(date, QTime(0,0).addSecs(issuingTime * 60), QTimeZone::UTC);
-    }
-    return QDateTime(date, {});
+    return FcbUtil::issuingDateTime(issuingYear, issuingDay, issuingTimeValue());
 }
 
-void Fcb::CustomerStatusType::decode(UPERDecoder &decoder)
+void Fcb::v13::CustomerStatusType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(statusProvider);
@@ -140,7 +79,7 @@ void Fcb::CustomerStatusType::decode(UPERDecoder &decoder)
     FCB_READ_IA5STRING(customerStatusDescr);
 }
 
-void Fcb::TravelerType::decode(UPERDecoder &decoder)
+void Fcb::v13::TravelerType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_UTF8STRING(firstName);
@@ -165,7 +104,7 @@ void Fcb::TravelerType::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CUSTOM(status);
 }
 
-void Fcb::TravelerData::decode(UPERDecoder &decoder)
+void Fcb::v13::TravelerData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_SEQUENCE_OF_CUSTOM(traveler);
@@ -173,12 +112,12 @@ void Fcb::TravelerData::decode(UPERDecoder &decoder)
     FCB_READ_UTF8STRING(groupName);
 }
 
-void Fcb::TrainLinkType::decode(UPERDecoder &decoder)
+void Fcb::v13::TrainLinkType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(train);
-    travelDate = decoder.readConstrainedWholeNumber(-1, 370);
-    departureTime = decoder.readConstrainedWholeNumber(0, 1440);
+    travelDate = decoder.readConstrainedWholeNumber(-1, FCB_DATE_MAX);
+    FCB_READ_TIME(departureTime);
     FCB_READ_CONSTRAINED_INT(departureUTCOffset, -60, 60);
     FCB_READ_INT_IA5_PAIR(fromStation, 1, 9999999);
     FCB_READ_INT_IA5_PAIR(toStation, 1, 9999999);
@@ -186,17 +125,12 @@ void Fcb::TrainLinkType::decode(UPERDecoder &decoder)
     FCB_READ_UTF8STRING(toStationNameUTF8);
 }
 
-QDateTime Fcb::TrainLinkType::departureDateTime(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::TrainLinkType::departureDateTime(const QDateTime &issueingDateTime) const
 {
-    QDate date = issueingDateTime.date().addDays(travelDate);
-    QTime time = QTime(0, 0).addSecs(departureTime * 60);
-    if (departureUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-departureUTCOffset * 15 * 60));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialTime(issueingDateTime, travelDate, departureTime, departureUTCOffsetValue());
 }
 
-void Fcb::ViaStationType::decode(UPERDecoder &decoder)
+void Fcb::v13::ViaStationType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_ENUM(stationCodeTable);
@@ -210,7 +144,7 @@ void Fcb::ViaStationType::decode(UPERDecoder &decoder)
     FCB_READ_UNCONSTRAINED_INT(routeId);
 }
 
-void Fcb::ZoneType::decode(UPERDecoder &decoder)
+void Fcb::v13::ZoneType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(carrier, 1, 32000);
@@ -223,7 +157,7 @@ void Fcb::ZoneType::decode(UPERDecoder &decoder)
     FCB_READ_IA5STRING(nutsCode);
 }
 
-void Fcb::LineType::decode(UPERDecoder &decoder)
+void Fcb::v13::LineType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(carrier, 1, 32000);
@@ -235,19 +169,19 @@ void Fcb::LineType::decode(UPERDecoder &decoder)
     FCB_READ_OCTETSTRING(binaryZoneId);
 }
 
-void Fcb::PolygoneType::decode(UPERDecoder &decoder)
+void Fcb::v13::PolygoneType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     firstEdge.decode(decoder);
     edges = decoder.readSequenceOf<DeltaCoordinate>();
 }
 
-void Fcb::RegionalValidityType::decode(UPERDecoder &decoder)
+void Fcb::v13::RegionalValidityType::decode(UPERDecoder &decoder)
 {
     value = decoder.readChoiceWithExtensionMarker<TrainLinkType, ViaStationType, ZoneType, LineType, PolygoneType>();
 }
 
-void Fcb::ReturnRouteDescriptionType::decode(UPERDecoder &decoder)
+void Fcb::v13::ReturnRouteDescriptionType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(fromStation, 1, 9999999);
@@ -258,7 +192,7 @@ void Fcb::ReturnRouteDescriptionType::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CUSTOM(validReturnRegion);
 }
 
-void Fcb::RouteSectionType::decode(UPERDecoder &decoder)
+void Fcb::v13::RouteSectionType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_ENUM(stationCodeTable);
@@ -268,7 +202,7 @@ void Fcb::RouteSectionType::decode(UPERDecoder &decoder)
     FCB_READ_UTF8STRING(toStationNameUTF8);
 }
 
-void Fcb::SeriesDetailType::decode(UPERDecoder &decoder)
+void Fcb::v13::SeriesDetailType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CONSTRAINED_INT(supplyingCarrier, 1, 32000);
@@ -276,7 +210,7 @@ void Fcb::SeriesDetailType::decode(UPERDecoder &decoder)
     FCB_READ_UNCONSTRAINED_INT(series);
 }
 
-void Fcb::CardReferenceType::decode(UPERDecoder &decoder)
+void Fcb::v13::CardReferenceType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(cardIssuer, 1, 32000);
@@ -287,7 +221,7 @@ void Fcb::CardReferenceType::decode(UPERDecoder &decoder)
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(trailingCardId);
 }
 
-void Fcb::PlacesType::decode(KItinerary::UPERDecoder &decoder)
+void Fcb::v13::PlacesType::decode(KItinerary::UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(coach);
@@ -297,7 +231,7 @@ void Fcb::PlacesType::decode(KItinerary::UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(placeNum, 1, 254);
 }
 
-void Fcb::CompartmentDetailsType::decode(UPERDecoder &decoder)
+void Fcb::v13::CompartmentDetailsType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CONSTRAINED_INT(coachType, 1, 99);
@@ -309,7 +243,7 @@ void Fcb::CompartmentDetailsType::decode(UPERDecoder &decoder)
     FCB_READ_ENUM(position);
 }
 
-void Fcb::BerthDetailData::decode(UPERDecoder &decoder)
+void Fcb::v13::BerthDetailData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     berthType = decoder.readEnumerated<BerthTypeType>();
@@ -317,7 +251,7 @@ void Fcb::BerthDetailData::decode(UPERDecoder &decoder)
     FCB_READ_ENUM(gender);
 }
 
-void Fcb::TariffType::decode(UPERDecoder &decoder)
+void Fcb::v13::TariffType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CONSTRAINED_INT(numberOfPassengers, 1, 200);
@@ -333,7 +267,7 @@ void Fcb::TariffType::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CUSTOM(reductionCard);
 }
 
-void Fcb::VatDetailType::decode(UPERDecoder &decoder)
+void Fcb::v13::VatDetailType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     country = decoder.readConstrainedWholeNumber(1, 999);
@@ -342,20 +276,20 @@ void Fcb::VatDetailType::decode(UPERDecoder &decoder)
     FCB_READ_IA5STRING(vatId);
 }
 
-void Fcb::IncludedOpenTicketType::decode(UPERDecoder &decoder)
+void Fcb::v13::IncludedOpenTicketType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_UNCONSTRAINED_INT(externalIssuerId);
     FCB_READ_UNCONSTRAINED_INT(issuerAutorizationId);
     FCB_READ_ENUM(stationCodeTable);
     FCB_READ_SEQUENCE_OF_CUSTOM(validRegion);
-    FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validFromDay, FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
     FCB_READ_ENUM(classCode);
     FCB_READ_IA5STRING_CONSTRAINED(serviceLevel, 1, 2);
@@ -368,7 +302,7 @@ void Fcb::IncludedOpenTicketType::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::RegisteredLuggageType::decode(UPERDecoder &decoder)
+void Fcb::v13::RegisteredLuggageType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(registrationId);
@@ -376,7 +310,7 @@ void Fcb::RegisteredLuggageType::decode(UPERDecoder &decoder)
     FCB_READ_CONSTRAINED_INT(maxSize, 1, 300);
 }
 
-void Fcb::LuggageRestrictionType::decode(UPERDecoder &decoder)
+void Fcb::v13::LuggageRestrictionType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CONSTRAINED_INT(maxHandLuggagePieces, 0, 99);
@@ -384,15 +318,15 @@ void Fcb::LuggageRestrictionType::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CUSTOM(registeredLuggage);
 }
 
-void Fcb::ReservationData::decode(UPERDecoder &decoder)
+void Fcb::v13::ReservationData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(train);
-    FCB_READ_CONSTRAINED_INT(departureDate, -1, 370);
+    FCB_READ_CONSTRAINED_INT(departureDate, -1, FCB_DATE_MAX);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_CONSTRAINED_INT(serviceBrand, 0, 32000);
     FCB_READ_UTF8STRING(serviceBrandAbrUTF8);
     FCB_READ_UTF8STRING(serviceBrandNameUTF8);
@@ -402,10 +336,10 @@ void Fcb::ReservationData::decode(UPERDecoder &decoder)
     FCB_READ_INT_IA5_PAIR(toStation, 1, 9999999);
     FCB_READ_UTF8STRING(fromStationNameUTF8);
     FCB_READ_UTF8STRING(toStationNameUTF8);
-    departureTime = decoder.readConstrainedWholeNumber(0, 1440);
+    FCB_READ_TIME(departureTime);
     FCB_READ_CONSTRAINED_INT(departureUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(arrivalDate, 0, 20);
-    FCB_READ_CONSTRAINED_INT(arrivalTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(arrivalDate, FCB_END_DATE_MIN, 20);
+    FCB_READ_TIME(arrivalTime);
     FCB_READ_CONSTRAINED_INT(arrivalUTCOffset, -60, 60);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(carrierNum, 1, 32000);
     FCB_READ_SEQUENCE_OF_IA5STRING(carrierIA5);
@@ -428,45 +362,28 @@ void Fcb::ReservationData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-QDateTime Fcb::ReservationData::departureDateTime(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::ReservationData::departureDateTime(const QDateTime &issueingDateTime) const
 {
-    QDate date = issueingDateTime.date().addDays(departureDate);
-    QTime time = QTime(0, 0).addSecs(departureTime * 60);
-    if (departureUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-departureUTCOffset * 15 * 60));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialTime(issueingDateTime, departureDate, departureTimeValue(), departureUTCOffsetValue());
 }
 
-QDateTime Fcb::ReservationData::arrivalDateTime(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::ReservationData::arrivalDateTime(const QDateTime &issueingDateTime) const
 {
-    if (!arrivalTimeIsSet()) {
-        return {};
-    }
-    const auto departureDt = departureDateTime(issueingDateTime);
-    QDate date = departureDt.date().addDays(arrivalDate);
-    QTime time = QTime(0, 0).addSecs(arrivalTime * 60);
-    if (arrivalUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-arrivalUTCOffset * 15 * 60));
-    }
-    if (departureDt.timeSpec() == Qt::OffsetFromUTC) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(departureDt.offsetFromUtc()));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialTime(departureDateTime(issueingDateTime), arrivalDate, arrivalTimeValue(), arrivalUTCOffsetValue());
 }
 
-void Fcb::CarCarriageReservationData::decode(UPERDecoder &decoder)
+void Fcb::v13::CarCarriageReservationData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(train);
-    FCB_READ_CONSTRAINED_INT(beginLoadingDate, -1, 370);
-    FCB_READ_CONSTRAINED_INT(beginLoadingTime, 0, 1440);
-    FCB_READ_CONSTRAINED_INT(endLoadingTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(beginLoadingDate, -1, FCB_DATE_MAX);
+    FCB_READ_TIME(beginLoadingTime);
+    FCB_READ_TIME(endLoadingTime);
     FCB_READ_CONSTRAINED_INT(loadingUTCOffset, -60, 60);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_CONSTRAINED_INT(serviceBrand, 0, 32000);
     FCB_READ_UTF8STRING(serviceBrandAbrUTF8);
     FCB_READ_UTF8STRING(serviceBrandNameUTF8);
@@ -500,12 +417,12 @@ void Fcb::CarCarriageReservationData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::OpenTicketData::decode(UPERDecoder &decoder)
+void Fcb::v13::OpenTicketData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(reference);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_UNCONSTRAINED_INT(extIssuerId);
     FCB_READ_UNCONSTRAINED_INT(issuerAutorizationId);
     returnIncluded = decoder.readBoolean();
@@ -517,13 +434,13 @@ void Fcb::OpenTicketData::decode(UPERDecoder &decoder)
     FCB_READ_UTF8STRING(validRegionDesc);
     FCB_READ_SEQUENCE_OF_CUSTOM(validRegion);
     FCB_READ_CUSTOM(returnDescription);
-    FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validFromDay, FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
-    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, 370);
+    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, FCB_DATE_MAX);
     FCB_READ_ENUM(classCode);
     FCB_READ_IA5STRING_CONSTRAINED(serviceLevel, 1, 2);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(carrierNum, 1, 32000);
@@ -539,75 +456,61 @@ void Fcb::OpenTicketData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-QDateTime Fcb::OpenTicketData::validFrom(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::OpenTicketData::validFrom(const QDateTime &issueingDateTime) const
 {
-    QDate date = issueingDateTime.date().addDays(validFromDay);
-    QTime time = validFromTimeIsSet() ? QTime(0, 0).addSecs(validFromTime * 60) : QTime();
-    if (validFromUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-validFromUTCOffset * 15 * 60));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialStartTime(issueingDateTime, validFromDay, validFromTimeValue(), validFromUTCOffsetValue());
 }
 
-QDateTime Fcb::OpenTicketData::validUntil(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::OpenTicketData::validUntil(const QDateTime &issueingDateTime) const
 {
-    const auto validFromDt = validFrom(issueingDateTime);
-    QDate date = validFromDt.date().addDays(validUntilDay);
-    QTime time = validUntilDayIsSet() ? QTime(0, 0).addSecs(validUntilTime * 60) : QTime(23, 59, 59);
-    if (validUntilUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-validUntilUTCOffset * 15 * 60));
-    }
-    if (validFromDt.timeSpec() == Qt::OffsetFromUTC) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(validFromDt.offsetFromUtc()));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialEndTime(validFrom(issueingDateTime), validUntilDay, validUntilTimeValue(), validFromUTCOffsetValue());
 }
 
-void Fcb::TimeRangeType::decode(UPERDecoder &decoder)
+void Fcb::v13::TimeRangeType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
-    fromTime = decoder.readConstrainedWholeNumber(0, 1440);
-    untilTime = decoder.readConstrainedWholeNumber(0, 1440);
+    FCB_READ_TIME(fromTime);
+    FCB_READ_TIME(untilTime);
 }
 
-void Fcb::ValidityPeriodType::decode(UPERDecoder &decoder)
+void Fcb::v13::ValidityPeriodType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
-    FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validFromDay, FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
 }
 
-void Fcb::ValidityPeriodDetailType::decode(UPERDecoder &decoder)
+void Fcb::v13::ValidityPeriodDetailType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_SEQUENCE_OF_CUSTOM(validityPeriod);
     FCB_READ_SEQUENCE_OF_CUSTOM(excludedTimeRange);
 }
 
-void Fcb::PassData::decode(UPERDecoder &decoder)
+void Fcb::v13::PassData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(reference);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_CONSTRAINED_INT(passType, 1, 250);
     FCB_READ_UTF8STRING(passDescription);
     FCB_READ_ENUM(classCode);
-    FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validFromDay, FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
     FCB_READ_CUSTOM(validityPeriodDetails);
-    FCB_READ_CONSTRAINED_INT(numberOfValidityDays, 0, 370);
+    FCB_READ_CONSTRAINED_INT(numberOfValidityDays, 0, FCB_DATE_MAX);
     FCB_READ_CONSTRAINED_INT(numberOfPossibleTrips, 1, 250);
     FCB_READ_CONSTRAINED_INT(numberOfDaysOfTravel, 1, 250);
-    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, 370);
+    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, FCB_DATE_MAX);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(countries, 1, 250);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(includedCarrierNum, 1, 32000);
     FCB_READ_SEQUENCE_OF_IA5STRING(includedCarrierIA5);
@@ -623,57 +526,43 @@ void Fcb::PassData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-QDateTime Fcb::PassData::validFrom(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::PassData::validFrom(const QDateTime &issueingDateTime) const
 {
-    QDate date = issueingDateTime.date().addDays(validFromDay);
-    QTime time = validFromTimeIsSet() ? QTime(0, 0).addSecs(validFromTime * 60) : QTime();
-    if (validFromUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-validFromUTCOffset * 15 * 60));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialStartTime(issueingDateTime, validFromDay, validFromTimeValue(), validFromUTCOffsetValue());
 }
 
-QDateTime Fcb::PassData::validUntil(const QDateTime &issueingDateTime) const
+QDateTime Fcb::v13::PassData::validUntil(const QDateTime &issueingDateTime) const
 {
-    const auto validFromDt = validFrom(issueingDateTime);
-    QDate date = validFromDt.date().addDays(validUntilDay);
-    QTime time = validUntilDayIsSet() ? QTime(0, 0).addSecs(validUntilTime * 60) : QTime(23, 59, 59);
-    if (validUntilUTCOffsetIsSet()) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(-validUntilUTCOffset * 15 * 60));
-    }
-    if (validFromDt.timeSpec() == Qt::OffsetFromUTC) {
-        return QDateTime(date, time, QTimeZone::fromSecondsAheadOfUtc(validFromDt.offsetFromUtc()));
-    }
-    return QDateTime(date, time);
+    return FcbUtil::decodeDifferentialEndTime(validFrom(issueingDateTime), validUntilDay, validUntilTimeValue(), validUntilUTCOffsetValue());
 }
 
-void Fcb::VoucherData::decode(UPERDecoder &decoder)
+void Fcb::v13::VoucherData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     validFromYear = decoder.readConstrainedWholeNumber(2016, 2269);
-    validFromDay = decoder.readConstrainedWholeNumber(0, 370);
+    validFromDay = decoder.readConstrainedWholeNumber(0, FCB_DATE_MAX);
     validUntilYear = decoder.readConstrainedWholeNumber(2016, 2269);
-    validUntilDay = decoder.readConstrainedWholeNumber(0, 370);
+    validUntilDay = decoder.readConstrainedWholeNumber(0, FCB_DATE_MAX);
     FCB_READ_UNCONSTRAINED_INT(value);
     FCB_READ_CONSTRAINED_INT(type, 1, 32000);
     FCB_READ_UTF8STRING(infoText);
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::CustomerCardData::decode(UPERDecoder &decoder)
+void Fcb::v13::CustomerCardData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CUSTOM(customer);
     FCB_READ_IA5STRING(cardIdIA5);
     FCB_READ_UNCONSTRAINED_INT(cardIdNum);
     validFromYear = decoder.readConstrainedWholeNumber(2016, 2269);
-    FCB_READ_CONSTRAINED_INT(validFromDay, 0, 370);
+    FCB_READ_CONSTRAINED_INT(validFromDay, 0, FCB_DATE_MAX);
     FCB_READ_CONSTRAINED_INT(validUntilYear, 0, 250);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, FCB_DATE_MAX);
     FCB_READ_ENUM(classCode);
     FCB_READ_CONSTRAINED_INT(cardType, 1, 1000);
     FCB_READ_UTF8STRING(cardTypeDescr);
@@ -683,32 +572,23 @@ void Fcb::CustomerCardData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-QDate Fcb::CustomerCardData::validFrom() const
+QDate Fcb::v13::CustomerCardData::validFrom() const
 {
-    QDate d(validFromYear, 1, 1);
-    if (validFromDayIsSet()) {
-        d = d.addDays(validFromDay);
-    }
-    return d;
+    return FcbUtil::decodeDate(validFromYear, validFromDayValue());
 }
 
-QDate Fcb::CustomerCardData::validUntil() const
+QDate Fcb::v13::CustomerCardData::validUntil() const
 {
-    QDate d(validFrom().year(), 1, 1);
-    d = d.addYears(validUntilYear);
-    if (validUntilDayIsSet()) {
-        d = d.addDays(validUntilDay);
-    }
-    return d;
+    return FcbUtil::decodeDifferentialDate(validFrom(), validUntilYear, validUntilDay);
 }
 
-void Fcb::CountermarkData::decode(UPERDecoder &decoder)
+void Fcb::v13::CountermarkData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_IA5STRING(ticketReferenceIA5);
     FCB_READ_UNCONSTRAINED_INT(ticketReferenceNum);
     numberOfCountermark = decoder.readConstrainedWholeNumber(1, 200);
@@ -723,11 +603,11 @@ void Fcb::CountermarkData::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_CUSTOM(validRegion);
     returnIncluded = decoder.readBoolean();
     FCB_READ_CUSTOM(returnDescription);
-    FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validFromDay, FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1440);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
     FCB_READ_ENUM(classCode);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(carrierNum, 1, 32000);
@@ -738,16 +618,16 @@ void Fcb::CountermarkData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::ParkingGroundData::decode(UPERDecoder &decoder)
+void Fcb::v13::ParkingGroundData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     parkingGroundId = decoder.readIA5String();
-    fromParkingDate = decoder.readConstrainedWholeNumber(-1, 370);
-    FCB_READ_CONSTRAINED_INT(untilParkingDate, 0, 370);
+    fromParkingDate = decoder.readConstrainedWholeNumber(FCB_BEGIN_DATE_MIN, 370);
+    FCB_READ_CONSTRAINED_INT(untilParkingDate, 0, FCB_DATE_MAX);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_IA5STRING(accessCode);
     location = decoder.readUtf8String();
     FCB_READ_ENUM(stationCodeTable);
@@ -761,16 +641,16 @@ void Fcb::ParkingGroundData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::FIPTicketData::decode(UPERDecoder &decoder)
+void Fcb::v13::FIPTicketData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_CONSTRAINED_INT(validFromDay, -1, 700);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, 370);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, FCB_DATE_MAX);
+    FCB_READ_SEQUENCE_OF_CONTRAINED_INT(activatedDay, 0, FCB_DATE_MAX);
     FCB_READ_SEQUENCE_OF_CONTRAINED_INT(carrierNum, 1, 32000);
     FCB_READ_SEQUENCE_OF_IA5STRING(carrierIA5);
     numberOfTravelDays = decoder.readConstrainedWholeNumber(1, 200);
@@ -779,13 +659,13 @@ void Fcb::FIPTicketData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::StationPassageData::decode(UPERDecoder &decoder)
+void Fcb::v13::StationPassageData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
     FCB_READ_UNCONSTRAINED_INT(referenceNum);
     FCB_READ_INT_IA5_PAIR(productOwner, 1, 32000);
-    FCB_READ_INT_IA5_PAIR(productId, 0, 32000);
+    FCB_READ_INT_IA5_PAIR(productId, 0, FCB_PRODUCT_ID_NUM_MAX);
     FCB_READ_UTF8STRING(productName);
     FCB_READ_ENUM(stationCodeTable);
     FCB_READ_SEQUENCE_OF_UNCONTRAINED_INT(stationNum);
@@ -794,17 +674,17 @@ void Fcb::StationPassageData::decode(UPERDecoder &decoder)
     FCB_READ_SEQUENCE_OF_UNCONTRAINED_INT(areaCodeNum);
     FCB_READ_SEQUENCE_OF_IA5STRING(areaCodeIA5);
     FCB_READ_SEQUENCE_OF_UTF8STRING(areaNameUTF8);
-    validFromDay = decoder.readConstrainedWholeNumber(-1, 700);
-    FCB_READ_CONSTRAINED_INT(validFromTime, 0, 1440);
+    validFromDay = decoder.readConstrainedWholeNumber(FCB_BEGIN_DATE_MIN, 700);
+    FCB_READ_TIME(validFromTime);
     FCB_READ_CONSTRAINED_INT(validFromUTCOffset, -60, 60);
-    FCB_READ_CONSTRAINED_INT(validUntilDay, 0, 370);
-    FCB_READ_CONSTRAINED_INT(validUntilTime, 0, 1400);
+    FCB_READ_CONSTRAINED_INT(validUntilDay, FCB_END_DATE_MIN, FCB_DATE_MAX);
+    FCB_READ_TIME(validUntilTime);
     FCB_READ_CONSTRAINED_INT(validUntilUTCOffset, -60, 60);
     FCB_READ_UNCONSTRAINED_INT(numberOfDaysValid);
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::DelayConfirmation::decode(UPERDecoder &decoder)
+void Fcb::v13::DelayConfirmation::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
@@ -812,7 +692,7 @@ void Fcb::DelayConfirmation::decode(UPERDecoder &decoder)
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(train);
     FCB_READ_CONSTRAINED_INT(departureYear, 2016, 2269);
     FCB_READ_CONSTRAINED_INT(departureDay, 1, 366);
-    FCB_READ_CONSTRAINED_INT(departureTime, 0, 1440);
+    FCB_READ_TIME(departureTime);
     FCB_READ_CONSTRAINED_INT(departureUTCOffset, -60, 60);
     FCB_READ_ENUM(stationCodeTable);
     FCB_READ_INT_IA5_PAIR(station, 1, 9999999);
@@ -824,7 +704,7 @@ void Fcb::DelayConfirmation::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-void Fcb::TokenType::decode(UPERDecoder &decoder)
+void Fcb::v13::TokenType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_INT_IA5_PAIR_UNCONSTRAINED(tokenProvider);
@@ -832,7 +712,7 @@ void Fcb::TokenType::decode(UPERDecoder &decoder)
     token = decoder.readOctetString();
 }
 
-void Fcb::DocumentData::decode(UPERDecoder &decoder)
+void Fcb::v13::DocumentData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_CUSTOM(token);
@@ -852,7 +732,7 @@ void Fcb::DocumentData::decode(UPERDecoder &decoder)
     >();
 }
 
-void Fcb::TicketLinkType::decode(UPERDecoder &decoder)
+void Fcb::v13::TicketLinkType::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_IA5STRING(referenceIA5);
@@ -864,7 +744,7 @@ void Fcb::TicketLinkType::decode(UPERDecoder &decoder)
     FCB_READ_ENUM(linkMode);
 }
 
-void Fcb::ControlData::decode(UPERDecoder &decoder)
+void Fcb::v13::ControlData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     FCB_READ_SEQUENCE_OF_CUSTOM(identificationByCardReference);
@@ -881,9 +761,9 @@ void Fcb::ControlData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(extension);
 }
 
-Fcb::UicRailTicketData::UicRailTicketData() = default;
+Fcb::v13::UicRailTicketData::UicRailTicketData() = default;
 
-Fcb::UicRailTicketData::UicRailTicketData(const Uic9183Block &block)
+Fcb::v13::UicRailTicketData::UicRailTicketData(const Uic9183Block &block)
     : m_block(block)
 {
     if (block.isNull()) {
@@ -897,7 +777,7 @@ Fcb::UicRailTicketData::UicRailTicketData(const Uic9183Block &block)
     }
 }
 
-void Fcb::UicRailTicketData::decode(UPERDecoder &decoder)
+void Fcb::v13::UicRailTicketData::decode(UPERDecoder &decoder)
 {
     decodeSequence(decoder);
     issuingDetail.decode(decoder);
@@ -906,7 +786,7 @@ void Fcb::UicRailTicketData::decode(UPERDecoder &decoder)
     FCB_READ_CUSTOM(controlDetail)
 }
 
-bool Fcb::UicRailTicketData::isValid() const
+bool Fcb::v13::UicRailTicketData::isValid() const
 {
     return !m_block.isNull();
 }
