@@ -4,8 +4,7 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#ifndef KITINERARY_IATABCBPDOCUMENTPROCESSOR_H
-#define KITINERARY_IATABCBPDOCUMENTPROCESSOR_H
+#pragma once
 
 #include <KItinerary/ExtractorDocumentProcessor>
 
@@ -23,4 +22,3 @@ public:
 
 }
 
-#endif // KITINERARY_IATABCBPDOCUMENTPROCESSOR_H
